#!/usr/bin/env python3
"""
 This should be able to analyze spectrum
* area
* gauss fit
And send data to influx (stored at ~/.seread.discover8086 ++ )
"""
from fire import Fire
#from tdb_io import influxwrite
import tdb_io

# let's see if we can fit
try:
    from pyfromroot import  prun
except:
    print("X... pyfrom root not imported.... ")

import datetime as dt
import time

from console import fg,bg



def get_ene_range(h):
    """
    input: TH1
    """
    blo = h.GetXaxis().GetFirst()
    bhi = h.GetXaxis().GetLast()
    a,b = get_calib_coefs(h)
    return a*blo + b, a*bhi+b

def get_calib_coefs(h):
    """
    input: TH1
    """
    bins = h.GetXaxis().GetNbins()
    xlo = h.GetXaxis().GetXmin()
    xhi = h.GetXaxis().GetXmax()
    # E = a*k +b
    a = (xhi-xlo)/bins
    b = xhi - a*bins
    return a,b


def bin2ene( binn, h):
    """
    input: TH1
    """
    a,b = get_calib_coefs( h )
    return a*binn+b

def ene2bin( ene, h ):
    """
    input: TH1
    """
    a,b = get_calib_coefs( h )
    return round((ene - b)/a)

def histo_unzoom(h):
    """
    input: TH1
    """
    # UnZoom() is crap. ... does everything....
    #histo_details(h)

    # this is going with bins, not energy
    bins = h.GetXaxis().GetNbins()
    h.GetXaxis().SetRange( 1, bins )
    #
    #h.GetXaxis().SetRangeUser( xlo, xhi ) # using energy

    #histo_details(h)
    return

def histo_area( h, elo=None, ehi=None ):
    """
    input: TH1
    """
    if elo is None:
        elo = h.GetXaxis().GetXmin() # energy, not bins GetFirst is bin
    if ehi is None:
        ehi = h.GetXaxis().GetXmax() # energy, not bins GetLast is bin
    intg = h.Integral( ene2bin(elo,h), ene2bin(ehi,h) )
    return intg

def histo_zoomenergy( h, xlo, xhi ):
    """
    input: TH1
    """
    print(f"i... zooming with energy  {xlo:.1f}...{xhi:.1f}" )
    intg = h.Integral( )
    #print(f"i... integral before zoom = {intg:16.6f}")
    h.GetXaxis().SetRangeUser( xlo, xhi ) # using energy
    intg = h.Integral( ene2bin(xlo,h), ene2bin(xhi,h) )
    #print(f"i... integral after  zoom = {intg:16.6f}")

    return

def histo_zoombins( h, xlo, xhi ):
    """
     input: TH1
    """
    print(f"i... zooming with bins {xlo:.1f}...{xhi:.1f}" )
    intg = h.Integral( )
    #print(f"i... integral before zoom = {intg}")
    h.GetXaxis().SetRange( xlo, xhi ) # using bins
    intg = h.Integral( xlo, xhi)
    #print(f"i... integral after zoom = {intg}")
    return

def histo_details( h ):
    """
    input: TH1
    Info needed for fit etc.
    """
    bins = h.GetXaxis().GetNbins()
    xlo = h.GetXaxis().GetXmin()
    xhi = h.GetXaxis().GetXmax()
    xzlo =h.GetXaxis().GetFirst()
    xzhi =h.GetXaxis().GetLast()
    intg = h.Integral()

    # E = a*k +b
    a,b = get_calib_coefs( h )
    #a = (xhi-xlo)/bins
    #b = xhi - a*bins

    print(f"i... | - bins={bins};  xmin-xmax = {xlo} - {xhi}")
    print(f"i... | - calib: a={a:9.6f}  b={b:7.3f}")
    print(f"i... | - zoom bins: {xzlo} ... {xzhi}")
    print(f"i... | - zoom ene : {a*xzlo+b:.1f} ... {a*xzhi+b:.1f}")
    print(f"i... | - integral={intg}  [of the zoomed area]")


def histo_fit( h ,  canvas = None, zoomme = None):
    """
    input: TH1
    """
    hname = h.GetName()
    print("i... fitting gauss in the range")

    # zoom?
    if "prun" in locals() or "prun" in globals() :
        res = prun.loadpy("fit",f"{hname} gpol1", canvas = canvas)
        if res['area']<0: res['area']=-res['area']
    else:
        print("X... pyfromroot NOT IMPORTED")
        res = None

    if abs(res['diff_fit_int_proc'])>1:
        print(f"X...  {fg.yellow}BAD DESCRIPTION - DIFF MORE THAN 1%{fg.default} ")
        time.sleep(0.2)

    if not( res['noerror']):
        print(f"X...  {fg.red}BAD DESCRIPTION - SOME ERROR OF FIT{fg.default}")
        time.sleep(0.2)
        return

    elif abs(res['area']) <= 2*res['darea']:
        print(f"X...  {fg.red}BAD DESCRIPTION - Area error > area/2{fg.default}")
        time.sleep(0.2)

    # for rate .... can by small
    # elif res['area'] <= 7:
    #     print(f"X...  {fg.red}BAD DESCRIPTION - SMALL Area < 7{fg.default}")
    #     time.sleep(0.2)

    else:
        now = dt.datetime.now()
        # wri = f"{(now-started).total_seconds():8.1f} {res['E']:9.2f} {res['dE']:8.3f} {res['area']:12.2f} {res['darea']:10.2f} {res['Efwhm']:10.3f} {res['dEfwhm']:7.2f}\n"
        # print(wri)
        cala, calb = get_calib_coefs( h )
        res["E"]       = res["channel"]*cala + calb
        res["dE"]      = res["dchannel"]*cala
        res["Efwhm"]   = res["fwhm"]*cala
        res["dEfwhm"]  = res["dfwhm"]*cala
        res["fittime"] = now
        return res
    return None



def main():
    print()

if __name__=="__main__":
    Fire(main)



#!/usr/bin/env python3

import sys

from fire import Fire
#------display
import ROOT
import numpy as np

from console import fg, bg, fx

import re # find livetime and a and b
from gregory_online.histo_global_cont import histograms

import math


def clear( h ):
    """
    histogram
    """
    for i in range( h.GetXaxis().GetNbins()+1 ):
        h.SetBinContent(i,0)
        h.SetBinError(i,0)
    return h



def get_range_from_calib(h,a,b):
    """
    avoid GetXmax ... it is a calib
    deduce really from max bin number
    """
    return a*0+b, a*h.GetXaxis().GetNbins()+b



def apply_calib(h1, xmin, xmax):
    if (xmin!=h1.GetXaxis().GetXmin() ) or (xmax!=h1.GetXaxis().GetXmax() ):
        print(f"i... calibration => {h1.GetXaxis().GetXmin()}->{xmin} ;{h1.GetXaxis().GetXmax()}->{xmax}")
        #print("i... limits before calib",xmin,xmax,h1.GetXaxis().GetXmin(),h1.GetXaxis().GetXmax())
        # calibrate ALL
        h1.GetXaxis().SetLimits(xmin,xmax)
        #print("i... limits after:",h1.GetXaxis().GetXmin(),h1.GetXaxis().GetXmax())




def get_np_from_h( h ):
    """
    return np
    """
    bins =  h.GetXaxis().GetNbins()
    hnp = np.zeros( bins )
    enp = np.zeros( bins )
    #print(len(hnp), hnp)
    for i in range(1, h.GetXaxis().GetNbins()+1 ): # no under, no over, last
        hnp[i-1] = h.GetBinContent(i)
        enp[i-1] = h.GetBinError(i)
        #if i<10: print("get",hnp[i-1],  enp[i-1])
    #print(hnp)
    return hnp,enp


def fill_h_with_np(h1np, hname="total" , runtime = None, errors=None, limits_calib=None, title = None):
    """
     histo treatment, histo is in gDirectory, treated by ROOT
    """
    global histograms
    mytitle = title

    if ROOT.gDirectory.FindObject(hname):
        #print(f"i... {hname} is there")
        histograms[hname] = ROOT.gDirectory.FindObject(hname)
    else:
        if mytitle is None: mytitle = hname
        print(f"{fg.cyan}X... histo={hname} is NOT there, we create now{fg.default}")
        histograms[hname] = ROOT.TH1D(hname,f"{hname}:{mytitle}", len(h1np)-2, 0, len(h1np)-2 )
        # print(f"i... {hname} is created now")

    suma = 0
    for i in range( len(h1np) ):
        suma+=h1np[i]
        histograms[hname].SetBinContent(i,h1np[i])
        if not errors is None:
            histograms[hname].SetBinError(i,errors[i]*0.999) # no y scale blinking
        else:
            histograms[hname].SetBinError(i, math.sqrt(h1np[i])*0.999 )# no y scale blinking
            #if i<10:  print( "fill",h1np[i], errors[i] )
        # self.h1_diff.SetBinContent(i, self.h1np_diff[i] )
        # #self.h1_dif1.SetBinContent(i, self.h1np_1mdiff[i] )
        # self.h1_dif2.SetBinContent(i, self.h1np_2mdiff[i] )
    histograms[hname].SetEntries(suma)

    if limits_calib is not None:
        xmin,xmax = limits_calib[0],limits_calib[1]
    else:
        xmin,xmax = 0,histograms[hname],GetXaxis().GetXmax() # this is the calib from h
    apply_calib(histograms[hname], xmin, xmax)


    #sqrt_done=False
    #if hname != "backg":  # do not repeat it for bg
    #    print("i... doing sumw2==0")
    #if errors is None:
        #histograms[hname].Sumw2(0) # THIS FOrces sqrt errors
    #     #sqrt_done = True
    #     #if sqrt_done:
    #     #print("i... doing sumw2==1")

        # CHECK THIS!!!!!!!!!!!!!!!!!!!!!!!!
    #    histograms[hname].Sumw2(1) # THIS LET IT TO BE SENSITIVE TO BackG and SCALE



    lowerror_cut = 0.6
    nonzero_mine = 0
    nonzero_min = 0
    if not(h1np is None) and  (len(h1np.nonzero())>1) and (h1np.nonzero()[0].min() >=0):
        if errors is None:
            nonzero_mine = np.sqrt( h1np.nonzero()[0].min() )*lowerror_cut
        else:
            nonzero_mine =  errors.nonzero()[0].min() * lowerror_cut

        # FIND nonezro minimum - error * factor=0.6
        if len(h1np.nonzero()[0])>0:
            nonzero_min = h1np.nonzero()[0].min() - nonzero_mine
        if  (runtime is not None) and (runtime>0.1): # SCALE ALSO ERRORS
            if len(h1np.nonzero()[0])>0 :
                nonzero_min = h1np.nonzero()[0].min()/runtime - nonzero_mine/runtime

    # ----------------- scale with runtime
    if h1np is not None and  runtime is not None:
        histograms[hname].Scale( 1/runtime)
        if len(h1np.nonzero()[0])>0 :
            if runtime>0: # crashed after server restart
                nonzero_min = h1np.nonzero()[0].min()/runtime - nonzero_mine/runtime



    # ------------ this prevents y-axis wobble in rate histogram
    binmx = histograms[hname].GetMaximumBin()
    binco = histograms[hname].GetBinContent( binmx)
    biner = histograms[hname].GetBinError( binmx)

    histomax = 1.01*(binco+biner)
    # histograms[hname].GetYaxis().GetXmin()
    #print(f"D... nonzero_min =   {nonzero_min}")
    #-----------I have a problem with that
    #histograms[hname].GetYaxis().SetRangeUser( nonzero_min, histomax )



def main():
    return

if __name__=="__main__":
    Fire(main)



#!/usr/bin/env python3
"""
display functions moved here
"""
from fire import Fire

from gregory_online.histo_global_cont import histograms
from gregory_online.histo_global_cont import canvases

from gregory_online.histo_analyze import get_calib_coefs

from console import fg, bg, fx
import os
import numpy as np


import datetime as dt

import pandas as pd

import ROOT #  for tests here with gDirectory ls
import time


# to fill if not displayable
from gregory_online.histo_np_ops import fill_h_with_np



def refresh_canvases():
    """
    unselectively refresh every canvas
    """
    maxc = ROOT.gROOT.GetListOfCanvases().GetEntries()
    for i in range(maxc):
        ci = ROOT.gROOT.GetListOfCanvases().At(i)
        ci.Modified()
        ci.Update()





def display_h_name( hname="tot", cname = "c_tot", filled = False , color = 39, title=None, simple=False):
    """
    display (e.g. substracted) histograms NEVER USED
    """
    global histograms,canvases
    if not  ROOT.gDirectory.FindObject(hname): # this naver happens
        print(f"i... histo {hname} is NOT found")
        return

    if ROOT.gDirectory.FindObject(cname):
        print(f"i... CANVAS {cname} is found")
        canvases[cname] = ROOT.gDirectory.FindObject(cname)
        canvases[cname].cd()
    else:
        print(f"{fg.cyan}w... CANVAS {cname} is NOT there, we {fg.green}create now{fg.default}")
        canvases[cname] = ROOT.TCanvas(cname,cname)
        canvases[cname].Draw()

    if ROOT.gDirectory.FindObject(hname): # this naver happens
        print(f"i... histo {hname} is found")

        #histograms[hname].SetLineColor(2)
        #histograms[hname].Draw("EX0")
        #histograms[hname].SetFillColor(19)
        histograms[hname].SetMarkerStyle(7)
        #histograms[hname].SetLineColor(16) # gray
        if filled:
            histograms[hname].SetFillStyle(3001)
            histograms[hname].SetFillColor(39)
            if simple:
                histograms[hname].Draw("")
            else:
                histograms[hname].Draw("hEX0")
        elif simple:
            histograms[hname].SetLineColor(color)
            histograms[hname].Draw("")
        else:
            histograms[hname].Draw("EX0")
        print("D... GRIDS HERE")
        ROOT.gPad.SetGridx()
        ROOT.gPad.SetGridy()
        ROOT.gPad.Modified()
        ROOT.gPad.Update()

    else:
        print(f"{fg.red}X... {hname} not found in gDirectory{fg.default}")
        ROOT.gDirectory.ls()
    #all reresh will be later
    #canvases[cname].SetGridx()
    #canvases[cname].SetGridy()
    canvases[cname].Modified()
    canvases[cname].Update()






def display_np(h1np, hname="tot", cname = "c_tot" , runtime=None, errors = None, limits_calib = None, filled = False, color = 16):
    """
    displaying needs a carefull treatment & histogram filling
    color 32, 48
    """
    global histograms,canvases
    if ROOT.gDirectory.FindObject(cname):
        print(f"i... CANVAS {cname} is found")
        canvases[cname] = ROOT.gDirectory.FindObject(cname)
        canvases[cname].cd()
    else:
        print(f"{fg.cyan}X... CANVAS {cname} is NOT there, we create now{fg.default}")
        canvases[cname] = ROOT.TCanvas(cname,cname)
        canvases[cname].Draw()

    if ROOT.gDirectory.FindObject(hname):
        print(f"i... histo {hname} is found")

        # histograms[hname].SetMarkerStyle(7)
        # histograms[hname].SetLineColor(16)
        # histograms[hname].Draw("EX0")

    else:
        #print(f"X... {hname} is NOT there, we run ")
        fill_h_with_np(h1np,hname, errors=errors, limits_calib=limits_calib, title = hname)
        #histograms[hname].Draw("HEX0") # E0 bars, X0 no 0,

    histograms[hname].SetMarkerStyle(7)
    histograms[hname].SetLineColor(color)
    if filled:
        histograms[hname].SetFillStyle(3001)
        histograms[hname].SetFillColor(color)
        histograms[hname].Draw("hEX0") # E0 bars, X0 no 0,
    else:
        histograms[hname].Draw("EX0") # E0 bars, X0 no 0,


    #all reresh will be later
    canvases[cname].Modified()
    canvases[cname].Update()




def main():
    print()

if __name__=="__main__":
    Fire(main)



#!/usr/bin/env python3
"""
This is a module that reads parquet (dcreated in nuphy2 from lara decay data
- prints a table
"""

from fire import Fire

import pandas as pd
from console import fg,bg
import importlib_resources # for data/*

from terminaltables import AsciiTable,SingleTable


PARQFILE = 'decay_lara.parquet'




def nicetime(t):
    suff = "s"
    if t>60:
        suff = "m"
        t = t/60
    if t>60:
        suff = "h"
        t = t/60
    if t>24:
        suff = "d"
        t = t/24
    if t>365:
        suff = "y"
        t = t/365

    if t>1e9:
        suff = "Gy"
        t = t/1e9
    if t>1e6:
        suff = "My"
        t = t/1e6
    if t>1e3:
        suff = "ky"
        t = t/1e3
    return f"{t:.2f} {suff}"



def candidates( dfo, e, de , t12low = 10, etrsh = 70, itrsh = 1):

    table = []
    print(f"i... {len(dfo)} gammas total", end="")
    dfo = dfo.loc[ dfo["E"]>etrsh ]

    df = dfo
    df = df.loc[ df["I"]>itrsh ]
    # df = df.loc[ df["E"]>etrsh ]
    df = df.loc[ df["T12"]>t12low ]
    print(f"; limited to {len(dfo)}: E>{etrsh}; I>{itrsh}; t12>{nicetime(t12low)}", end="")

    df = df.loc[ (df["E"]>e-de) & (df["E"]<e+de) ]
    print(f"; {len(df)} candidate(s) for dE={de:.2f}")#; limited E>{etrsh}; I>{itrsh}; t12>{nicetime(t12low)}")
    #print(f"i... {len(df)} candidates; limited E>{etrsh}; I>{itrsh}; t12>{nicetime(t12low)}")
    # susp = list(df['name'].values.tolist())


    table_data = []

    column=-1
    trow = -1
    shift = 30
    #table_data.append([]) # row 0

    for index, row in df.iterrows():
        column+=1
        currint = row['I']
        currene = row['E']
        currname  = row['name']
        ss = f"{fg.white}{currname}    {nicetime(row['T12'])} {fg.default}"
        table_data.append( [ss] )

        dfsel = dfo.loc[ (dfo['name']==currname) & (dfo['I']>=currint/3) ]
        if len(dfsel)>2:
            dfsel = dfsel.sort_values( ['I','E'] , ascending = False)

        trow = 0
        enepres = False
        mute = False
        for i2,r2 in dfsel.iterrows():
            trow+=1
            if trow>7: # thick to display not more 7, but add ene
                mute = True # demutes if green
            ss = ""
            if r2['E']==currene:
                ss+=f"{fg.green}"
                enepres = True
                mute = False
            else:
                ss+=f"{fg.default}"
            ss+= f"{r2['E']:8.2f}  {r2['I']:7.2f}"
            #if r2['E']==currene:
            ss+=f"{fg.default}"
            if not mute:
                table_data[-1].append( ss )
            #print(table_data)


    print( )
    #for i in range(len(table_data)):
    #    print(table_data[i])

    # before transposition, I need full square
    mx = 0
    for i in table_data:
        if mx<len(i):mx=len(i)
    #print(mx)
    for i in table_data:
        while len(i)<mx:i.append("")

    if len(table_data)>0:
        table_data = list(map(lambda *x: list(x), *table_data)) # transpose
    SingleTable( table_data )
    #print(table_data)

    table_instance = SingleTable(table_data, "candidates")
    # table_instance.justify_columns[2] = 'right'
    if column <6:
        print(table_instance.table)
    else:
        print(f"i... too many columns ({column+1})")





def decay_candidates( E, dE=1):
    """
    verify the parquet
    """
    #E = 121

    ret = importlib_resources.files("gregory_online").joinpath("data/"+PARQFILE)
    #print(ret)
    dfo = pd.read_parquet(ret)
    # dfo = dfo.loc[ dfo["E"]>70 ]

    candidates( dfo , E, dE)

    #pd.set_option("display.max_rows", None, "display.max_columns", 7)
    #df = df.sort_values( ["E","A"] )


    #print(df)

#def main():
    # load()



if __name__=="__main__":
    Fire( decay_candidates   )



#!/usr/bin/env python3
#
# FROM READ_VME_OFFLINE ----------------
#
from fire import Fire
# from read_vme_offline.version import __version__

import configparser
from math import exp

import matplotlib.pyplot as plt
import matplotlib.ticker as plticker

import sys
import os

from console import fg,bg
#print("!!!...  USE READ_VME_OFFLINE FROM 2022/07 ...... ")
#sys.exit(1)

CROW = 0


def is_float(n): # FROM FLASHCAM
    try:
        float_n = float(n)
    except ValueError:
        return False
    else:
        return True


def get_sections( configfile = 'dpp_pha.ini' ):
    """
    returns list of sections. Check if COMMON or channel present
    """
    config = configparser.ConfigParser( delimiters=(" ","\t") )
    config.read( configfile)
    return config.sections()


def get_setting( config, parameter, channel=0, retfloat = True):
    """
    search parameter like TRG_HOLDOFF in configparser - first in [CHAN] then in COMMON
    normally return float
    """
    ch = f"{channel}"
    res = None
    if  not ch in config:
        print(f"X... {fg.red}channel {channel} NOT in config {fg.default}")
        return None
    if parameter in config[ch]:
        res = config[ ch ][parameter]
        print(f"D... {parameter:20s} @ch {channel} ...",res)
    else:
        res = config[f'COMMON'][parameter]
        print(f"D... {parameter:20s} @comm ...",res)
    if retfloat:
        if is_float(res) :
            return float(res)
        else:
            print("X... {fg.red}NOT A FLOAT NUMBER{fg.default}", res)
            return None
    return None


def open_parser( inifile, channel ):
    config = configparser.ConfigParser( delimiters=(" ","\t") )
    config.read( inifile)

    print("D... SECTIONS:",config.sections() )
    if len(config.sections())==0:
        print("X... NO SECTIONS FOUND IN ",inifile)
        return None

    if channel is not None:
        if not str(channel) in config:
            print(f"X... channel {channel} {bg.red}     NOT PRESENT     {bg.default}")
            return None
        else:
            print(f"D... channel {channel} IS PRESENT")
    return config


def extract_holdoffs( inifile, channel):
    config = open_parser( inifile, channel)
    if config is None: return 0,0,0
    TRG_HOLDOFF      = get_setting( config, 'TRG_HOLDOFF'      ,channel  )
    TF_PEAK_HOLDOFF  = get_setting( config, 'TF_PEAK_HOLDOFF'  ,channel  )
    TF_SHAPING_TIME  = get_setting( config, 'TF_SHAPING_TIME'  ,channel  )
    TF_SEL_PEAKMEAN  = get_setting( config, 'TF_SEL_PEAKMEAN'  ,channel  )
    TF_PEAKING_DELAY = get_setting( config, 'TF_PEAKING_DELAY' ,channel  )
    TF_FLAT_TOP      = get_setting( config, 'TF_FLAT_TOP'      ,channel  )
    ECALa            = get_setting( config, 'CALIBRATION_A'    ,channel  )
    ECALb            = get_setting( config, 'CALIBRATION_B'    ,channel  )

    WID=0.01
    if TF_SEL_PEAKMEAN==0:WID=0.01
    elif TF_SEL_PEAKMEAN==1:WID=0.01*4
    elif TF_SEL_PEAKMEAN==2:WID=0.01*16
    elif TF_SEL_PEAKMEAN==3:WID=0.01*64
    else: WID=3

    # i dont knwow why 0.16 is here
    TWIN_US = round(0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY + WID,1)
    PKHOLD  = round(0.16+   TF_SHAPING_TIME+TF_FLAT_TOP +TF_PEAK_HOLDOFF )

    TAG = f"{bg.green}{fg.white}"
    if TWIN_US==0 or TRG_HOLDOFF==0 or PKHOLD==0:
        f"{bg.red}{fg.white}"
    print(f"{TAG}TRG_HOLDOFF     {TRG_HOLDOFF:5.1f} us{bg.default}{fg.default}" )
    # print(f"{TAG}TF_PEAK_HOLDOFF {TF_PEAK_HOLDOFF:5.1f}{bg.default}{fg.default}" )
    print(f"{TAG}PKHOLD          {PKHOLD:5.1f} us{bg.default}{fg.default}" )
    print(f"{TAG}TWIN_US         {TWIN_US:5.1f} us{bg.default}{fg.default}" )


    return TRG_HOLDOFF, PKHOLD, TWIN_US, ECALa, ECALb


def main( inifile = 'dpp_pha.ini', channel=None):
    """ Make jpg image from INI config file - timing
    """
    global ACROW
    # config = configparser.ConfigParser( delimiters=(" ","\t") )
    # config.read( inifile)

    # print("D... SECTIONS:",config.sections() )
    # if len(config.sections())==0:
    #     print("X... NO SECTIONS FOUND IN ",inifile)
    #     return

    # if channel is not None:
    #     if not str(channel) in config:
    #         print(f"X... channel {channel} {fg.red} NOT PRESENT")
    #         return
    #     else:
    #         print(f"D... channel {channel} IS PRESENT")
    extract_holdoffs(inifile, channel)

    config = open_parser( inifile, channel)
    if config is None: return
    #=============everything will be in microseconds
    STEP=0.01

    #CALA = get_setting( config, "CALIBRATION_A", channel )
    #RECORD_LENGTH=int(config['COMMON']['RECORD_LENGTH'])*STEP
    RECORD_LENGTH    = get_setting( config, "RECORD_LENGTH", channel )*STEP
    PRE_TRIGGER      = get_setting( config, 'PRE_TRIGGER', channel )*STEP
    RISE_TIME        = get_setting( config, 'RISE_TIME' ,channel  )
    TF_DECAY_TAU     = get_setting( config, 'TF_DECAY_TAU' ,channel  )
    TF_SHAPING_TIME  = get_setting( config, 'TF_SHAPING_TIME' ,channel  )
    TF_FLAT_TOP      = get_setting( config, 'TF_FLAT_TOP' ,channel  )
    TF_PEAKING_DELAY = get_setting( config, 'TF_PEAKING_DELAY' ,channel  )
    TF_SEL_PEAKMEAN  = get_setting( config, 'TF_SEL_PEAKMEAN' ,channel  )
    TRG_HOLDOFF      = get_setting( config, 'TRG_HOLDOFF' ,channel  )
    TF_PEAK_HOLDOFF  = get_setting( config, 'TF_PEAK_HOLDOFF' ,channel  )
    RT_DISCR_WINDOW  = get_setting( config, 'RT_DISCR_WINDOW' ,channel  )
    ENABLE_RT_DISCR  = get_setting( config, 'ENABLE_RT_DISCR', channel , retfloat = False)




    if ENABLE_RT_DISCR=="NO":
        ENABLE_RT_DISCR = False
    else:
        ENABLE_RT_DISCR = True

    # it seems that it includes the pretrigger..........
    RECORD_LENGTH = RECORD_LENGTH - PRE_TRIGGER #


    plt.figure( figsize=(10,14))


    loc = plticker.MultipleLocator( base= 0.5 )


    ROWS=11
    ACROW = 0
    def incac():  # jump to next panel
        global ACROW
        ACROW+=1
        return ACROW

    INP=plt.subplot( ROWS, 1,incac())
    plt.setp(INP.get_xticklabels(), visible=False )
    INP.xaxis.set_major_locator(loc)
    INP.grid()
    INP.text(1.0,0.5,'INPUT',
            horizontalalignment='right',
            transform=INP.transAxes)

    DEL=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(DEL.get_xticklabels(), visible=False)
    DEL.xaxis.set_major_locator(loc)
    DEL.grid()
    DEL.text(1.0,.5,'DELTA',
            horizontalalignment='right',
            transform=DEL.transAxes)

    DEL2=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(DEL2.get_xticklabels(), visible=False)
    DEL2.xaxis.set_major_locator(loc)
    DEL2.grid()
    DEL2.text(1.0,.5,'DELTA2',
            horizontalalignment='right',
            transform=DEL2.transAxes)

    ARM=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(ARM.get_xticklabels(), visible=False)
    ARM.xaxis.set_major_locator(loc)
    ARM.grid()
    ARM.text(1.0,.5,'ARMED',
            horizontalalignment='right',
            transform=ARM.transAxes)

    TRG=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(TRG.get_xticklabels(), visible=False)
    TRG.grid()
    TRG.text(1.0,.5,'TRG_REQ',
            horizontalalignment='right',
            transform=TRG.transAxes)

    RTD=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(RTD.get_xticklabels(), visible=False)
    RTD.grid()
    RTD.text(1.0,.5,'RTDwin: acceptance win for TRG_REQ, discr.pileups',
            horizontalalignment='right',
            transform=RTD.transAxes)

    TRGHO=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(TRGHO.get_xticklabels(), visible=False)
    TRGHO.grid()
    TRGHO.text(1.0,.5,'TRG_HOLDOFF: prevent TRG gen on overshoot',
            horizontalalignment='right',
            transform=TRGHO.transAxes)

    TRAP=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(TRAP.get_xticklabels(), visible=False)
    TRAP.grid()
    TRAP.text(1.0,.5,'TRAPEZOID',
            horizontalalignment='right',
            transform=TRAP.transAxes)


    PEAK=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(PEAK.get_xticklabels(), visible=False)
    PEAK.grid()
    PEAK.text(1.0,.5,'PEAKING',
            horizontalalignment='right',
            transform=PEAK.transAxes)



    PEAKHO=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(PEAKHO.get_xticklabels(), visible=False)
    PEAKHO.grid()
    PEAKHO.text(1.0, 0.5 ,'PEAK_HOLDOFF',
            horizontalalignment='right',
            transform=PEAKHO.transAxes)



    PKRUN=plt.subplot(ROWS,1,incac(),sharex = INP)
    plt.setp(PKRUN.get_xticklabels(), visible=True)
    PKRUN.grid()
    PKRUN.text(1.0,.5,'PEAKRUN',
            horizontalalignment='right',
            transform=PKRUN.transAxes)


    # TXTF=plt.subplot(10,1,1,sharex = INP)
    # plt.setp(TXTF.get_xticklabels(), visible=True)
    # #TXTF.grid()


    # TXTF.text(1.0, 0.5 ,'info',
    #         horizontalalignment='right',
    #         transform=TXTF.transAxes)
    # TXTF.text(0.0, 0. ,'TRG_HOLDOFF ... we tried 0.4, but 0.8 made a great improvement with 6MeV - no pileups',
    #         horizontalalignment='left',
    #         transform=TXTF.transAxes)
    # TXTF.text(0.0, 0.25 ,'TRG_HOLDOFF ... we tried 0.4, but 0.8 made a great improvement with 6MeV - no pileups',
    #         horizontalalignment='left',
    #         transform=TXTF.transAxes)
    # TXTF.text(0.0, 0.5 ,'TRG_HOLDOFF ... we tried 0.4, but 0.8 made a great improvement with 6MeV - no pileups',
    #         horizontalalignment='left',
    #         transform=TXTF.transAxes)
    # TXTF.text(0.0, 0.75 ,'TRG_HOLDOFF ... we tried 0.4, but 0.8 made a great improvement with 6MeV - no pileups',
    #         horizontalalignment='left',
    #         transform=TXTF.transAxes)




    #========================================================== PLOTS=========
    # make a little extra space between the subplots
    #fig.subplots_adjust(hspace=0.5)

    #INP.set_xlim(0, RECORD_LENGTH)
    INP.set_xlim(-PRE_TRIGGER, RECORD_LENGTH)

    #ax2.set_xlim(0, RECORD_LENGTH)
    #ax1.get_shared_x_axes().join(ax1, ax2)
    #ax1 = plt.subplot(gs[1], sharex = ax0)
    #ax2.grid()



    #==============================  INPUT CALC
    t=RECORD_LENGTH-PRE_TRIGGER
    t=RECORD_LENGTH
    INP_AT_RC=exp(-t/TF_DECAY_TAU)
    #INP.plot([0,PRE_TRIGGER,PRE_TRIGGER+ RISE_TIME, RECORD_LENGTH],[0,0, 1 , INP_AT_RC],'b.-')
    INP.plot([-PRE_TRIGGER, -RISE_TIME,0 , RECORD_LENGTH],[0,0, 1 , INP_AT_RC],'b.-')
    #plt.subplots_adjust(hspace=.0)

    #==============================   DELTA
    #t=[0,PRE_TRIGGER, PRE_TRIGGER+ RISE_TIME,PRE_TRIGGER+ RISE_TIME*2, RECORD_LENGTH]
    t=[-PRE_TRIGGER, -RISE_TIME, 0 ,0+ RISE_TIME, RECORD_LENGTH]
    s=[0, 0,           1 ,                     0,                      0]
    DEL.plot(t,s,'g.-')
    #plt.subplots_adjust(hspace=.0)
    T_TRIGGER=RISE_TIME


    #==============================  DELTA2
    t=[0,PRE_TRIGGER, PRE_TRIGGER+ T_TRIGGER/2,
       PRE_TRIGGER+ T_TRIGGER,
       PRE_TRIGGER+ T_TRIGGER*1.5,
       PRE_TRIGGER+ T_TRIGGER*2,
       RECORD_LENGTH]

    t=[-PRE_TRIGGER,
       -RISE_TIME, -3*RISE_TIME/4, -RISE_TIME/2,
       0,  RISE_TIME/2,
       RISE_TIME, RECORD_LENGTH]
    s=[0,
       0,  0.5, 1 ,
       0, -1,
       0,0]
    DEL2.plot(t,s,'g.-')
    plt.subplots_adjust(hspace=.0)


    #==============================  ARMED
    THRESHOLD = -3*RISE_TIME/4
    t=[-PRE_TRIGGER,
       -RISE_TIME, THRESHOLD, THRESHOLD,  # threshold here
       0,  TRG_HOLDOFF, TRG_HOLDOFF,
        RECORD_LENGTH]
    s=[0,
       0,  0,1,
       1, 1, 0,
       0]
    ARM.plot(t,s,'m.-')
    plt.subplots_adjust(hspace=.0)


    #========================================--TRG-REQ
    t=[0, PRE_TRIGGER+T_TRIGGER,PRE_TRIGGER+T_TRIGGER,PRE_TRIGGER+T_TRIGGER,RECORD_LENGTH]
    t=[0, 0+T_TRIGGER,0+T_TRIGGER,0+T_TRIGGER,RECORD_LENGTH]
    t=[-PRE_TRIGGER, 0,0,0,RECORD_LENGTH]
    s=[0,0,1,0,0]
    TRG.plot(t,s,'r-*' )


    #========================================--RTDwin
    t=[-PRE_TRIGGER,
       THRESHOLD, THRESHOLD,  # at -rise3/4 ~
       THRESHOLD+RT_DISCR_WINDOW,THRESHOLD+RT_DISCR_WINDOW,
       RECORD_LENGTH]
    s=[0,
       0,1,
       1,0,
       0]
    if ENABLE_RT_DISCR:
        RTD.plot(t,s,'r-*' )
    else:
        RTD.plot(t,s,':.', color='gray' )



    #========================================--TRGHO
    t=[0,
          PRE_TRIGGER+T_TRIGGER, PRE_TRIGGER+T_TRIGGER,
          PRE_TRIGGER+T_TRIGGER+TRG_HOLDOFF,PRE_TRIGGER+T_TRIGGER+TRG_HOLDOFF,
          RECORD_LENGTH]
    t=[-PRE_TRIGGER,
          0, 0,
          TRG_HOLDOFF,TRG_HOLDOFF,
          RECORD_LENGTH]
    s=[0,
       0, 1,
       1, 0,
      0]
    TRGHO.plot(t,s,'m*-' )

    #========================================--TRAP  16 clock cycles delay after trgreq
    t=[0, PRE_TRIGGER+T_TRIGGER+0.16,
          PRE_TRIGGER+T_TRIGGER+0.16 + TF_SHAPING_TIME,
          PRE_TRIGGER+T_TRIGGER+0.16 + TF_SHAPING_TIME+TF_FLAT_TOP,
          RECORD_LENGTH]
    t=[-PRE_TRIGGER, 0.16,
          0.16 + TF_SHAPING_TIME,
          0.16 + TF_SHAPING_TIME+TF_FLAT_TOP,
          0.16 + 2*TF_SHAPING_TIME+TF_FLAT_TOP,
          RECORD_LENGTH]
    s=[0,0,
       1,
       1,
       0,
      0]
    TRAP.plot(t,s,'g.-' )


    #========================================--PEAK
    WID=0.01
    if TF_SEL_PEAKMEAN==0:WID=0.01
    elif TF_SEL_PEAKMEAN==1:WID=0.01*4
    elif TF_SEL_PEAKMEAN==2:WID=0.01*16
    elif TF_SEL_PEAKMEAN==3:WID=0.01*64
    else: WID=3
    t=[-PRE_TRIGGER,
          PRE_TRIGGER+T_TRIGGER+0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY,
          PRE_TRIGGER+T_TRIGGER+0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY,
          PRE_TRIGGER+T_TRIGGER+0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY+WID,
          PRE_TRIGGER+T_TRIGGER+0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY+WID,
          RECORD_LENGTH]
    t=[-PRE_TRIGGER,
          0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY,
          0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY,
          0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY + WID,
          0.16 + TF_SHAPING_TIME+TF_PEAKING_DELAY + WID,
          RECORD_LENGTH]
    s=[0,
       0, 1,
       1, 0,
      0]
    PEAK.plot(t,s,'r-*' )




    #========================================--PEAKHOLDOFF
    t=[-PRE_TRIGGER,
          0.16+   TF_SHAPING_TIME+TF_FLAT_TOP ,
          0.16+   TF_SHAPING_TIME+TF_FLAT_TOP ,
          0.16+   TF_SHAPING_TIME+TF_FLAT_TOP +TF_PEAK_HOLDOFF ,
          0.16+   TF_SHAPING_TIME+TF_FLAT_TOP +TF_PEAK_HOLDOFF ,
          RECORD_LENGTH]
    s=[0,
       0, 1,
       1, 0,
      0]
    PEAKHO.plot(t,s,'g*-' )


    #========================================--PEAKRUN
    t=[-PRE_TRIGGER,
          0, 0,
          0.16+   TF_SHAPING_TIME+TF_FLAT_TOP ,
          0.16+   TF_SHAPING_TIME+TF_FLAT_TOP +TF_PEAK_HOLDOFF ,
          0.16+   TF_SHAPING_TIME+TF_FLAT_TOP  +TF_PEAK_HOLDOFF,
          RECORD_LENGTH]
    s=[0,
       0, 1,
       1,
       1,
       0,
      0]
    PKRUN.plot(t,s,'m*-' )

    plt.xlabel('t [us]', fontsize=18)
    OUTFILE =  "dpp_pha.ini.jpg"
    OUTFILE = os.path.splitext( inifile )[0] + ".png"
    plt.savefig(OUTFILE, bbox_inches='tight')
    plt.show()





if __name__=="__main__":
    Fire(main)
